@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-help.c,v 1.2.2.1 2006/04/10 09:37:25 pborelli Exp $
+ * $Id: gedit-commands-help.c,v 1.6 2006/04/20 10:40:46 sfre Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -82,7 +82,8 @@ gedit_cmd_help_about (GtkAction   *action,
 
 	gedit_debug (DEBUG_COMMANDS);
 
-	logo = gdk_pixbuf_new_from_file (GNOME_ICONDIR "/gedit-logo.png", NULL);
+	logo = gdk_pixbuf_new_from_file (GEDIT_DATADIR "/logo/gedit-logo.png",
+					 NULL);
 
 	gtk_show_about_dialog (GTK_WINDOW (window),
 			       "authors", authors,
