@@ -24,9 +24,9 @@ from SnippetPlaceholders import *
 from SnippetComplete import SnippetComplete
 
 class SnippetController:
-	TAB_KEY_VAL = (gtk.gdk.keyval_from_name('Tab'), \
-			gtk.gdk.keyval_from_name('ISO_Left_Tab'))
-	SPACE_KEY_VAL = (gtk.gdk.keyval_from_name('space'),)
+	TAB_KEY_VAL = (gtk.keysyms.Tab, \
+			gtk.keysyms.ISO_Left_Tab)
+	SPACE_KEY_VAL = (gtk.keysyms.space,)
 	
 	def __init__(self, instance, view):
 		self.view = None
@@ -34,25 +34,29 @@ class SnippetController:
 		
 		self.placeholders = []
 		self.active_snippets = []
+		self.active_placeholder = None
 		self.signal_ids = {}
 		
 		self.update_placeholders = []
+		self.jump_placeholders = []
 		self.language_name = 0
-		self.idle_update_id = 0
+		self.timeout_update_id = 0
 		
 		self.set_view(view)
 	
 	# Stop controlling the view. Remove all active snippets, remove references
 	# to the view and the plugin instance, disconnect all signal handlers
 	def stop(self):
-		if self.idle_update_id != 0:
-			gobject.source_remove(self.idle_update_id)
-			self.idle_update_id = 0
+		if self.timeout_update_id != 0:
+			gobject.source_remove(self.timeout_update_id)
+			self.timeout_update_id = 0
 			del self.update_placeholders[:]
+			del self.jump_placeholders[:]
 
 		SnippetsLibrary().unref(None)
 		self.set_view(None)
 		self.instance = None
+		self.active_placeholder = None
 
 	def disconnect_signal(self, obj, signal):
 		if signal in self.signal_ids:
@@ -123,6 +127,8 @@ class SnippetController:
 		snippets = SnippetsLibrary().from_accelerator(accelerator, \
 				self.language_name)
 
+		snippets_debug('Accel!')
+
 		if len(snippets) == 0:
 			return False
 		elif len(snippets) == 1:
@@ -130,7 +136,7 @@ class SnippetController:
 		else:
 			# Do the fancy completion dialog
 			return self.show_completion(snippets)
-	
+
 		return True
 
 	def first_snippet_inserted(self):
@@ -146,11 +152,26 @@ class SnippetController:
 		self.disconnect_signal(buf, 'changed')
 		self.disconnect_signal(buf, 'cursor-moved')
 
+	def current_placeholder(self):
+		buf = self.view.get_buffer()
+		
+		piter = buf.get_iter_at_mark(buf.get_insert())	
+		current = None
+
+		for placeholder in self.placeholders:
+			begin = placeholder.begin_iter()
+			end = placeholder.end_iter()
+
+			if piter.compare(begin) >= 0 and \
+					piter.compare(end) <= 0:
+				current = placeholder
+
+		return current
+
 	def advance_placeholder(self, direction):
 		# Returns (CurrentPlaceholder, NextPlaceholder), depending on direction
 		buf = self.view.get_buffer()
 		
-		# Find out if we are in a placeholder now
 		piter = buf.get_iter_at_mark(buf.get_insert())
 		prev = current = next = None
 		length = len(self.placeholders)		
@@ -211,6 +232,8 @@ class SnippetController:
 			if current.__class__ == SnippetPlaceholderEnd:
 				last = current
 		
+		self.active_placeholder = next
+		
 		if next:
 			next.enter()
 			
@@ -220,7 +243,7 @@ class SnippetController:
 		if last:
 			# This is the end of the placeholder, remove the snippet etc
 			for snippet in list(self.active_snippets):
-				if snippet[2][0] == last:
+				if snippet[3][0] == last:
 					self.deactivate_snippet(snippet)
 					break
 		
@@ -236,27 +259,55 @@ class SnippetController:
 		(current, prev) = self.previous_placeholder()
 		return self.goto_placeholder(current, prev)
 
-	def update_environment(self):
-		buf = self.view.get_buffer()
+	def env_get_selected_text(self, buf):
 		bounds = buf.get_selection_bounds()
-		
+
 		if bounds:
-			os.environ['GEDIT_SELECTED_TEXT'] = buf.get_text(bounds[0], \
-					bounds[1])
+			return buf.get_text(bounds[0], bounds[1])
 		else:
-			os.environ['GEDIT_SELECTED_TEXT'] = ''
+			return ''
+
+	def env_get_current_word(self, buf):
+		start, end = buffer_word_boundary(buf)
 		
+		return buf.get_text(start, end)
+		
+	def env_get_filename(self, buf):
 		uri = buf.get_uri()
 		
 		if uri:
-			os.environ['GEDIT_FILENAME'] = \
-					buf.get_uri_for_display()
-			os.environ['GEDIT_BASENAME'] = \
-					os.path.basename(uri)
+			return buf.get_uri_for_display()
 		else:
-			os.environ['GEDIT_FILENAME'] = ''
-			os.environ['GEDIT_BASENAME'] = ''
+			return ''
+	
+	def env_get_basename(self, buf):
+		uri = buf.get_uri()
 		
+		if uri:
+			return os.path.basename(uri)
+		else:
+			return ''
+
+	def update_environment(self):
+		buf = self.view.get_buffer()
+		
+		variables = {'GEDIT_SELECTED_TEXT': self.env_get_selected_text, \
+				'GEDIT_CURRENT_WORD': self.env_get_current_word, \
+				'GEDIT_FILENAME': self.env_get_filename, \
+				'GEDIT_BASENAME': self.env_get_basename}
+		
+		for var in variables:
+			os.environ[var] = variables[var](buf)
+	
+	def uses_current_word(self, snippet):
+		matches = re.findall('(\\\\*)\\$GEDIT_CURRENT_WORD', snippet['text'])
+		
+		for match in matches:
+			if len(match) % 2 == 0:
+				return True
+		
+		return False
+
 	def apply_snippet(self, snippet, start = None, end = None):
 		buf = self.view.get_buffer()
 		s = Snippet(snippet)
@@ -266,7 +317,13 @@ class SnippetController:
 		
 		if not end:
 			end = buf.get_iter_at_mark(buf.get_selection_bound())
-		
+
+		if start.equal(end) and self.uses_current_word(s):
+			# There is no tab trigger and no selection and the snippet uses
+			# the current word. Set start and end to the word boundary so that 
+			# it will be removed
+			start, end = buffer_word_boundary(buf)
+
 		# Set environmental variables
 		self.update_environment()
 		
@@ -276,7 +333,8 @@ class SnippetController:
 			self.goto_placeholder(current, None)
 		
 		buf.begin_user_action()
-		# Remove the tag
+
+		# Remove the tag, selection or current word
 		buf.delete(start, end)
 		
 		# Insert the snippet
@@ -359,25 +417,31 @@ class SnippetController:
 		buf = self.view.get_buffer()
 		remove = []
 		
-		for tabstop in snippet[2]:
+		for tabstop in snippet[3]:
 			if tabstop == -1:
-				placeholders = snippet[2][-1]
+				placeholders = snippet[3][-1]
 			else:
-				placeholders = [snippet[2][tabstop]]
+				placeholders = [snippet[3][tabstop]]
 			
 			for placeholder in placeholders:
 				if placeholder in self.placeholders:
 					if placeholder in self.update_placeholders:
 						placeholder.update_contents()
+					elif placeholder in self.jump_placeholders:
+						placeholder[0].leave()
 						
 					remove.append(placeholder)
 		
 		for placeholder in remove:
+			if placeholder == self.active_placeholder:
+				self.active_placeholder = None
+
 			self.placeholders.remove(placeholder)
 			placeholder.remove(force)
-				
+
 		buf.delete_mark(snippet[0])
 		buf.delete_mark(snippet[1])
+		buf.delete_mark(snippet[2])
 		
 		self.active_snippets.remove(snippet)
 		
@@ -466,12 +530,17 @@ class SnippetController:
 		return complete.run()
 
 	def update_snippet_contents(self):
-		self.idle_update_id = 0
+		self.timeout_update_id = 0
 		
 		for placeholder in self.update_placeholders:
 			placeholder.update_contents()
 		
+		for placeholder in self.jump_placeholders:
+			self.goto_placeholder(placeholder[0], placeholder[1])
+		
 		del self.update_placeholders[:]
+		del self.jump_placeholders[:]
+		
 		return False
 
 	# Callbacks
@@ -503,15 +572,32 @@ class SnippetController:
 				if piter.compare(begin) < 0 or piter.compare(end) > 0:
 					# Oh no! Remove the snippet this instant!!
 					self.deactivate_snippet(snippet)
+		
+		current = self.current_placeholder()
+		
+		if current != self.active_placeholder:
+			if self.active_placeholder:
+				self.jump_placeholders.append((self.active_placeholder, current))
+				
+				if self.timeout_update_id != 0:
+					gobject.source_remove(self.timeout_update_id)
+				
+				self.timeout_update_id = gobject.timeout_add(0, 
+						self.update_snippet_contents)
+
+			self.active_placeholder = current
 	
 	def on_buffer_changed(self, buf):
-		current, next = self.advance_placeholder(1)
+		current = self.current_placeholder()
 		
-		if current and current.needs_updating():
+		if current:
 			self.update_placeholders.append(current)
 		
-			if self.idle_update_id == 0:
-				self.idle_update_id = gobject.idle_add(self.update_snippet_contents)
+			if self.timeout_update_id != 0:
+				gobject.source_remove(self.timeout_update_id)
+
+			self.timeout_update_id = gobject.timeout_add(0, \
+					self.update_snippet_contents)
 	
 	def on_notify_language(self, buf, spec):
 		self.update_language()
@@ -529,5 +615,11 @@ class SnippetController:
 				not (event.state & gdk.SHIFT_MASK) and \
 				event.keyval in self.SPACE_KEY_VAL:
 			return self.show_completion()
+		elif not SnippetsLibrary().loaded and \
+				valid_accelerator(event.keyval, event.state):
+			SnippetsLibrary().ensure_files()
+			SnippetsLibrary().ensure(self.language_name)
+			self.accelerator_activate(event.keyval, \
+					event.state & gtk.accelerator_get_default_mod_mask())
 
 		return False
